@@ -1,5 +1,7 @@
 use regex::Regex;
 use std::ffi::{OsStr, OsString};
+use std::fs::File;
+use std::io::BufRead;
 use std::ops::{Div, Mul};
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
@@ -32,12 +34,35 @@ pub struct Cli {
     pub output: Vec<PathBuf>,
 
     /// Genome size to calculate coverage with respect to. e.g., 4.3kb, 7Tb, 9000, 4.1MB
-    #[structopt(short = "g", long = "genome-size")]
-    pub genome_size: GenomeSize,
+    ///
+    /// Alternatively, a FASTA/Q index file can be provided and the genome size will be
+    /// set to the sum of all reference sequences.
+    ///
+    /// If --bases is not provided, this option and --coverage are required
+    #[structopt(
+        short = "g",
+        long,
+        required_unless = "bases",
+        value_name = "size|faidx"
+    )]
+    pub genome_size: Option<GenomeSize>,
 
-    /// The desired coverage to sub-sample the reads to.
-    #[structopt(short = "c", long = "coverage", value_name = "FLOAT")]
-    pub coverage: Coverage,
+    /// The desired coverage to sub-sample the reads to
+    ///
+    /// If --bases is not provided, this option and --genome-size are required
+    #[structopt(
+        short = "c",
+        long = "coverage",
+        value_name = "FLOAT",
+        required_unless = "bases"
+    )]
+    pub coverage: Option<Coverage>,
+
+    /// Explicitly set the number of bases required e.g., 4.3kb, 7Tb, 9000, 4.1MB
+    ///
+    /// If this option is given, --coverage and --genome-size are ignored
+    #[structopt(short = "b", long = "bases", value_name = "bases", required_unless_all = &["coverage", "genome-size"])]
+    pub bases: Option<GenomeSize>,
 
     /// Random seed to use.
     #[structopt(short = "s", long = "seed", value_name = "INT")]
@@ -114,6 +139,10 @@ pub enum CliError {
     /// Indicates a bad combination of input and output files was passed.
     #[error("Bad combination of input and output files: {0}")]
     BadInputOutputCombination(String),
+
+    /// Faidx IO error
+    #[error("Failed to open/parse faidx file {0}")]
+    FaidxError(String),
 }
 
 /// A metric suffix is a unit suffix used to indicate the multiples of (in this case) base pairs.
@@ -197,6 +226,12 @@ impl PartialEq<u64> for GenomeSize {
     }
 }
 
+impl From<GenomeSize> for u64 {
+    fn from(g: GenomeSize) -> Self {
+        g.0
+    }
+}
+
 impl FromStr for GenomeSize {
     type Err = CliError;
 
@@ -214,9 +249,10 @@ impl FromStr for GenomeSize {
         let re = Regex::new(r"(?P<size>[0-9]*\.?[0-9]+)(?P<sfx>\w*)$").unwrap();
         let captures = match re.captures(text.as_str()) {
             Some(cap) => cap,
-            None => {
-                return Err(CliError::InvalidGenomeSizeString(s.to_string()));
-            }
+            None => match Self::from_faidx(Path::new(&s)) {
+                Ok(g) => return Ok(g),
+                _ => return Err(CliError::InvalidGenomeSizeString(s.to_string())),
+            },
         };
         let size = captures
             .name("size")
@@ -230,6 +266,23 @@ impl FromStr for GenomeSize {
     }
 }
 
+impl GenomeSize {
+    fn from_faidx(path: &Path) -> Result<Self, CliError> {
+        let mut size = 0;
+        let file = File::open(path)
+            .map_err(|_| CliError::FaidxError(path.to_string_lossy().to_string()))?;
+        let rdr = std::io::BufReader::new(file);
+        for result in rdr.lines() {
+            let line =
+                result.map_err(|_| CliError::FaidxError(path.to_string_lossy().to_string()))?;
+            let fields: Vec<&str> = line.split('\t').collect();
+            size += u64::from_str(fields[1])
+                .map_err(|_| CliError::FaidxError(path.to_string_lossy().to_string()))?;
+        }
+        Ok(GenomeSize(size))
+    }
+}
+
 /// Allow for multiplying a `GenomeSize` by a [`Coverage`](#coverage).
 ///
 /// # Example
@@ -452,6 +505,36 @@ mod tests {
         assert_eq!(actual, expected)
     }
 
+    #[test]
+    fn no_genome_size_but_bases() {
+        let infile = "tests/cases/r1.fq.gz";
+        let passed_args = vec!["rasusa", "-i", infile, "-b", "5", "-c", "7"];
+
+        let args = Cli::from_iter_safe(passed_args).unwrap();
+
+        assert_eq!(args.bases.unwrap(), GenomeSize(5));
+    }
+
+    #[test]
+    fn bases_and_coverage_and_genome_size_all_llowed() {
+        let infile = "tests/cases/r1.fq.gz";
+        let passed_args = vec!["rasusa", "-i", infile, "-b", "5", "-c", "7", "-g", "5m"];
+
+        let args = Cli::from_iter_safe(passed_args).unwrap();
+
+        assert_eq!(args.bases.unwrap(), GenomeSize(5));
+    }
+
+    #[test]
+    fn no_genome_size_or_coverage_given_but_bases_given() {
+        let infile = "tests/cases/r1.fq.gz";
+        let passed_args = vec!["rasusa", "-i", infile, "-b", "5"];
+
+        let args = Cli::from_iter_safe(passed_args).unwrap();
+
+        assert_eq!(args.bases.unwrap(), GenomeSize(5));
+    }
+
     #[test]
     fn invalid_genome_size_given_raises_error() {
         let passed_args = vec!["rasusa", "-i", "in.fq", "-c", "5", "-g", "8jb"];
@@ -463,6 +546,16 @@ mod tests {
         assert_eq!(actual, expected)
     }
 
+    #[test]
+    fn faidx_given_as_genome_size() {
+        let infile = "tests/cases/r1.fq.gz";
+        let faidx = "tests/cases/h37rv.fa.fai";
+        let passed_args = vec!["rasusa", "-i", infile, "-c", "5", "-g", faidx];
+        let args = Cli::from_iter_safe(passed_args).unwrap();
+
+        assert_eq!(args.genome_size.unwrap(), GenomeSize(4411532))
+    }
+
     #[test]
     fn invalid_seed_given_raises_error() {
         let passed_args = vec!["rasusa", "-i", "in.fq", "-c", "5", "-g", "8mb", "-s", "foo"];
@@ -493,8 +586,8 @@ mod tests {
         let args = Cli::from_iter_safe(passed_args).unwrap();
 
         assert_eq!(args.input[0], PathBuf::from_str(infile).unwrap());
-        assert_eq!(args.coverage, Coverage(5.0));
-        assert_eq!(args.genome_size, GenomeSize(8_000_000));
+        assert_eq!(args.coverage.unwrap(), Coverage(5.0));
+        assert_eq!(args.genome_size.unwrap(), GenomeSize(8_000_000));
         assert_eq!(args.seed, Some(88));
         assert_eq!(
             args.output[0],
@@ -526,8 +619,8 @@ mod tests {
             PathBuf::from_str(infile).unwrap(),
         ];
         assert_eq!(args.input, expected_input);
-        assert_eq!(args.coverage, Coverage(5.0));
-        assert_eq!(args.genome_size, GenomeSize(8_000_000));
+        assert_eq!(args.coverage.unwrap(), Coverage(5.0));
+        assert_eq!(args.genome_size.unwrap(), GenomeSize(8_000_000));
         assert_eq!(args.seed, Some(88));
         assert_eq!(
             args.output[0],
@@ -560,8 +653,8 @@ mod tests {
             PathBuf::from_str(infile).unwrap(),
         ];
         assert_eq!(args.input, expected_input);
-        assert_eq!(args.coverage, Coverage(5.0));
-        assert_eq!(args.genome_size, GenomeSize(8_000_000));
+        assert_eq!(args.coverage.unwrap(), Coverage(5.0));
+        assert_eq!(args.genome_size.unwrap(), GenomeSize(8_000_000));
         assert_eq!(args.seed, Some(88));
         assert_eq!(
             args.output[0],
@@ -775,6 +868,32 @@ mod tests {
         assert!(actual.is_err());
     }
 
+    #[test]
+    fn genome_size_from_faidx() {
+        let p = Path::new("tests/cases/h37rv.fa.fai");
+        let actual = GenomeSize::from_faidx(p).unwrap();
+        let expected = GenomeSize(4411532);
+
+        assert_eq!(actual, expected)
+    }
+
+    #[test]
+    fn genome_size_from_faidx_fastq_index() {
+        let p = Path::new("tests/cases/file1.fq.fai");
+        let actual = GenomeSize::from_faidx(p).unwrap();
+        let expected = GenomeSize(10050);
+
+        assert_eq!(actual, expected)
+    }
+
+    #[test]
+    fn genome_size_from_str_fastq_index() {
+        let actual = GenomeSize::from_str("tests/cases/file1.fq.fai").unwrap();
+        let expected = GenomeSize(10050);
+
+        assert_eq!(actual, expected)
+    }
+
     #[test]
     fn int_coverage_returns_float() {
         let actual = Coverage::from_str("56").unwrap();
