@@ -1,6 +1,6 @@
+use crate::cli::CompressionExt;
 use needletail::errors::ParseErrorKind::EmptyFile;
 use needletail::parse_fastx_file;
-use std::collections::HashSet;
 use std::fs::File;
 use std::io::{BufWriter, Write};
 use std::path::{Path, PathBuf};
@@ -29,6 +29,10 @@ pub enum FastxError {
     #[error("Output file could not be created")]
     CreateError { source: std::io::Error },
 
+    /// Indicates and error trying to create the compressor
+    #[error(transparent)]
+    CompressOutputError(#[from] niffler::Error),
+
     /// Indicates that some indices we expected to find in the input file weren't found.
     #[error("Some expected indices were not in the input file")]
     IndicesNotFound,
@@ -71,15 +75,23 @@ impl Fastx {
     /// let path = std::path::Path::new("output.fa");
     /// let fastx = Fastx{ path };
     /// { // this scoping means the file handle is closed afterwards.
-    ///     let file_handle = fastx.create()?;
+    ///     let file_handle = fastx.create(6, None)?;
     ///     write!(file_handle, ">read1\nACGT\n")?
     /// }
     /// ```
-    pub fn create(&self) -> Result<Box<dyn Write>, FastxError> {
+    pub fn create(
+        &self,
+        compression_lvl: niffler::compression::Level,
+        compression_fmt: Option<niffler::compression::Format>,
+    ) -> Result<Box<dyn Write>, FastxError> {
         let file = File::create(&self.path).map_err(|source| FastxError::CreateError { source })?;
-        let file_handle = BufWriter::new(file);
-
-        Ok(Box::new(file_handle))
+        let file_handle = Box::new(BufWriter::new(file));
+        let fmt = match compression_fmt {
+            None => niffler::Format::from_path(&self.path),
+            Some(f) => f,
+        };
+        niffler::get_writer(file_handle, fmt, compression_lvl)
+            .map_err(FastxError::CompressOutputError)
     }
 
     /// Returns a vector containing the lengths of all the reads in the file.
@@ -135,12 +147,12 @@ impl Fastx {
     /// let mut input = tempfile::Builder::new().suffix(".fastq").tempfile().unwrap();
     /// input.write_all(text.as_bytes()).unwrap();
     /// let fastx = Fastx::from_path(input.path()).unwrap();
-    /// let mut reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![1]);
+    /// let mut reads_to_keep: Vec<bool> = vec![false, true]);
     /// let output = Builder::new().suffix(".fastq").tempfile().unwrap();
     /// let output_fastx = Fastx::from_path(output.path()).unwrap();
     /// {
     ///     let mut out_fh = output_fastx.create().unwrap();
-    ///     let filter_result = fastx.filter_reads_into(&mut reads_to_keep, &mut out_fh);
+    ///     let filter_result = fastx.filter_reads_into(&mut reads_to_keep, 1, &mut out_fh);
     ///     assert!(filter_result.is_ok());
     /// }
     /// let actual = std::fs::read_to_string(output).unwrap();
@@ -149,36 +161,37 @@ impl Fastx {
     /// ```
     pub fn filter_reads_into<T: Write>(
         &self,
-        mut reads_to_keep: HashSet<u32>,
+        reads_to_keep: &[bool],
+        nb_reads_keep: usize,
         write_to: &mut T,
     ) -> Result<usize, FastxError> {
         let mut total_len = 0;
         let mut reader =
             parse_fastx_file(&self.path).map_err(|source| FastxError::ReadError { source })?;
-        let mut read_idx: u32 = 0;
-        while let Some(record) = reader.next() {
-            let keep_this_read = reads_to_keep.contains(&read_idx);
-            read_idx += 1; // increment here so we don't need to do it twice below
+        let mut read_idx: usize = 0;
+        let mut nb_reads_written = 0;
 
+        while let Some(record) = reader.next() {
             match record {
                 Err(source) => return Err(FastxError::ParseError { source }),
-                Ok(rec) if keep_this_read => {
+                Ok(rec) if reads_to_keep[read_idx] => {
                     total_len += rec.num_bases();
                     rec.write(write_to, None)
                         .map_err(|err| FastxError::WriteError {
                             source: anyhow::Error::from(err),
-                        })?
+                        })?;
+                    nb_reads_written += 1;
+                    if nb_reads_keep == nb_reads_written {
+                        break;
+                    }
                 }
-                Ok(_) => continue,
+                Ok(_) => (),
             }
 
-            reads_to_keep.remove(&(read_idx - 1)); // remember we already incremented the index
-
-            if reads_to_keep.is_empty() {
-                return Ok(total_len);
-            }
+            read_idx += 1;
         }
-        if reads_to_keep.is_empty() {
+
+        if nb_reads_written == nb_reads_keep {
             Ok(total_len)
         } else {
             Err(FastxError::IndicesNotFound)
@@ -190,9 +203,7 @@ impl Fastx {
 mod tests {
     use super::*;
     use std::any::Any;
-    use std::collections::HashSet;
     use std::io::{Read, Write};
-    use std::iter::FromIterator;
     use std::path::Path;
     use tempfile::Builder;
 
@@ -212,7 +223,10 @@ mod tests {
     fn create_invalid_output_file_raises_error() {
         let path = Path::new("invalid/out/path.fq");
 
-        let actual = Fastx::from_path(path).create().err().unwrap();
+        let actual = Fastx::from_path(path)
+            .create(niffler::Level::Eight, None)
+            .err()
+            .unwrap();
         let expected = FastxError::CreateError {
             source: std::io::Error::new(
                 std::io::ErrorKind::Other,
@@ -226,7 +240,9 @@ mod tests {
     #[test]
     fn create_valid_output_file_and_can_write_to_it() {
         let file = Builder::new().suffix(".fastq").tempfile().unwrap();
-        let mut writer = Fastx::from_path(file.path()).create().unwrap();
+        let mut writer = Fastx::from_path(file.path())
+            .create(niffler::Level::Eight, None)
+            .unwrap();
 
         let actual = writer.write(b"foo\nbar");
 
@@ -236,7 +252,9 @@ mod tests {
     #[test]
     fn create_valid_compressed_output_file_and_can_write_to_it() {
         let file = Builder::new().suffix(".fastq.gz").tempfile().unwrap();
-        let mut writer = Fastx::from_path(file.path()).create().unwrap();
+        let mut writer = Fastx::from_path(file.path())
+            .create(niffler::Level::Four, None)
+            .unwrap();
 
         let actual = writer.write(b"foo\nbar");
 
@@ -288,11 +306,11 @@ mod tests {
         let mut input = Builder::new().suffix(".fastq").tempfile().unwrap();
         input.write_all(text.as_bytes()).unwrap();
         let fastx = Fastx::from_path(input.path());
-        let reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![]);
+        let reads_to_keep: Vec<bool> = vec![false];
         let output = Builder::new().suffix(".fastq").tempfile().unwrap();
         let output_fastx = Fastx::from_path(output.path());
-        let mut out_fh = output_fastx.create().unwrap();
-        let filter_result = fastx.filter_reads_into(reads_to_keep, &mut out_fh);
+        let mut out_fh = output_fastx.create(niffler::Level::Four, None).unwrap();
+        let filter_result = fastx.filter_reads_into(&reads_to_keep, 0, &mut out_fh);
 
         assert!(filter_result.is_ok());
 
@@ -309,12 +327,12 @@ mod tests {
         let mut input = Builder::new().suffix(".fastq").tempfile().unwrap();
         input.write_all(text.as_bytes()).unwrap();
         let fastx = Fastx::from_path(input.path());
-        let reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![0]);
+        let reads_to_keep: Vec<bool> = vec![true];
         let output = Builder::new().suffix(".fastq").tempfile().unwrap();
         let output_fastx = Fastx::from_path(output.path());
         {
-            let mut out_fh = output_fastx.create().unwrap();
-            let filter_result = fastx.filter_reads_into(reads_to_keep, &mut out_fh);
+            let mut out_fh = output_fastx.create(niffler::Level::Four, None).unwrap();
+            let filter_result = fastx.filter_reads_into(&reads_to_keep, 1, &mut out_fh);
             assert!(filter_result.is_ok());
         }
 
@@ -330,12 +348,12 @@ mod tests {
         let mut input = Builder::new().suffix(".fa").tempfile().unwrap();
         input.write_all(text.as_bytes()).unwrap();
         let fastx = Fastx::from_path(input.path());
-        let reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![0]);
+        let reads_to_keep: Vec<bool> = vec![true];
         let output = Builder::new().suffix(".fa").tempfile().unwrap();
         let output_fastx = Fastx::from_path(output.path());
         {
-            let mut out_fh = output_fastx.create().unwrap();
-            let filter_result = fastx.filter_reads_into(reads_to_keep, &mut out_fh);
+            let mut out_fh = output_fastx.create(niffler::Level::Four, None).unwrap();
+            let filter_result = fastx.filter_reads_into(&reads_to_keep, 1, &mut out_fh);
             assert!(filter_result.is_ok());
         }
 
@@ -351,12 +369,12 @@ mod tests {
         let mut input = Builder::new().suffix(".fastq").tempfile().unwrap();
         input.write_all(text.as_bytes()).unwrap();
         let fastx = Fastx::from_path(input.path());
-        let reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![1]);
+        let reads_to_keep: Vec<bool> = vec![false, true];
         let output = Builder::new().suffix(".fastq").tempfile().unwrap();
         let output_fastx = Fastx::from_path(output.path());
         {
-            let mut out_fh = output_fastx.create().unwrap();
-            let filter_result = fastx.filter_reads_into(reads_to_keep, &mut out_fh);
+            let mut out_fh = output_fastx.create(niffler::Level::Four, None).unwrap();
+            let filter_result = fastx.filter_reads_into(&reads_to_keep, 1, &mut out_fh);
             assert!(filter_result.is_ok());
         }
 
@@ -372,12 +390,12 @@ mod tests {
         let mut input = Builder::new().suffix(".fastq").tempfile().unwrap();
         input.write_all(text.as_bytes()).unwrap();
         let fastx = Fastx::from_path(input.path());
-        let reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![0, 2]);
+        let reads_to_keep: Vec<bool> = vec![true, false, true];
         let output = Builder::new().suffix(".fastq").tempfile().unwrap();
         let output_fastx = Fastx::from_path(output.path());
         {
-            let mut out_fh = output_fastx.create().unwrap();
-            let filter_result = fastx.filter_reads_into(reads_to_keep, &mut out_fh);
+            let mut out_fh = output_fastx.create(niffler::Level::Four, None).unwrap();
+            let filter_result = fastx.filter_reads_into(&reads_to_keep, 2, &mut out_fh);
             assert!(filter_result.is_ok());
         }
 
@@ -393,12 +411,12 @@ mod tests {
         let mut input = Builder::new().suffix(".fa").tempfile().unwrap();
         input.write_all(text.as_bytes()).unwrap();
         let fastx = Fastx::from_path(input.path());
-        let reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![0, 2]);
+        let reads_to_keep: Vec<bool> = vec![true, false, true];
         let output = Builder::new().suffix(".fa").tempfile().unwrap();
         let output_fastx = Fastx::from_path(output.path());
         {
-            let mut out_fh = output_fastx.create().unwrap();
-            let filter_result = fastx.filter_reads_into(reads_to_keep, &mut out_fh);
+            let mut out_fh = output_fastx.create(niffler::Level::Four, None).unwrap();
+            let filter_result = fastx.filter_reads_into(&reads_to_keep, 2, &mut out_fh);
             assert!(filter_result.is_err());
         }
 
@@ -414,12 +432,12 @@ mod tests {
         let mut input = Builder::new().suffix(".fq").tempfile().unwrap();
         input.write_all(text.as_bytes()).unwrap();
         let fastx = Fastx::from_path(input.path());
-        let reads_to_keep: HashSet<u32> = HashSet::from_iter(vec![0, 2]);
+        let reads_to_keep: Vec<bool> = vec![true, false, true];
         let output = Builder::new().suffix(".fq").tempfile().unwrap();
         let output_fastx = Fastx::from_path(output.path());
         {
-            let mut out_fh = output_fastx.create().unwrap();
-            let filter_result = fastx.filter_reads_into(reads_to_keep, &mut out_fh);
+            let mut out_fh = output_fastx.create(niffler::Level::Four, None).unwrap();
+            let filter_result = fastx.filter_reads_into(&reads_to_keep, 2, &mut out_fh);
             assert!(filter_result.is_err());
         }
 
